/************************************************
*
* JCH2k I2C Library for simultanious master & slave operation
*
* (c) 2006, 2007 J.Claudius Hagen, www.jch2k.de
*
* Revision          : $Revision: 610 $
* Date              : $Date: 2007-03-07 01:51:26 +0100 (Mi, 07 Mrz 2007) $
* Created at        : June 2007
* Updated by        : $Author: claudius $
*
* Details: http://techtalk.jch2k.de (german)
*
**************************************************/

#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "twi_masterslave.h"
#include "uart.h"


#define CMD_PC_POWER_SET    0x10
#define CMD_PC_POWER_GET    0x20
#define CMD_UART_PUT        0x55

// protos
void toggle_relais (void);
uint8_t computer_state(void);
void start_system_timer(void);

void oszi(uint8_t bit);

uint8_t action = 0;
uint8_t master = 0;
uint8_t slave = 0;


// ========================
// COMPUTER CONTROL
//
void toggle_relais ()
{
	PORTD |= (1<<PD2);
	
	OCR1A = 10000; //= 62500 (*256 = 16M);
	TCNT1 = 0;
	TCCR1A = 0; // Timer reset on Compare Match
	TCCR1B = (1<<CS12); // Prescaler /256 --> start timer!
	TIMSK |= (1<<OCIE1A); // enable interrupt
}

// release relais
ISR(TIMER1_COMPA_vect)
{
	PORTD &= ~(1<<PD2);
	
	TCCR1B = 0;
}

uint8_t computer_state(void)
{
	return !(PIND & (1<<PD6));
}

//===========
// Debugging helper

void oszi(uint8_t bit)
{
	PORTC = (1<<bit);
	for (int i=0; i<5; i++)
		asm volatile ("nop");
	PORTC = 0;
}


// ======================
// SYSTEM TIMER for UART
//
void start_system_timer()
{
	// WGM -> phase correct pwm mode
	// CS  -> prescaler to /64 --> ca. 500Hz @ 8MHz clock
	TCCR0 = (1<<CS01) | (1<<CS00);
	
	// enable interrupts
	TIMSK |= (1<<TOIE0);
}

ISR (TIMER0_OVF_vect) {
	
	// check the UART for new data
	unsigned int rec = uart_getc();
  if (rec != UART_NO_DATA)
  {
  	switch ((unsigned char)rec)
  	{
  		case 'a':
  			toggle_relais();
  			break;
  			
  		case 'b':
			  uart_putc ('b');
			  
			  twiBuffer[0] = ((0x10|slave)<<1);
			  twiBuffer[1] = 0x20;
			  twiStart(2);
			  break;
  	} 
  }
  
  
  // this is the "revenge"
  if (action)
  {
  	action++;
  	
  	if (action == 5)
  	{
  		twiBuffer[0] = ((0x10|slave)<<1);
		  twiBuffer[1] = 0x20;
		  
		  twiStart(2);
  		
  		action = 0;
  	}
  }  
}



// ==================
// CALLBACK ROUTINES

void mrxByteReceived(uint8_t b)
{
	// nothing has to be done
	// but you can handle the received byte if you're in a hurry
}

void srxByteReceived(uint8_t b)
{
	// nothing has to be done
	// but you can handle the received byte if you're in a hurry
}

void gcxByteReceived(uint8_t b)
{
	// nothing has to be done
	// but you can handle the received byte if you're in a hurry
}


void stxStarted(void)
{
	// here you got some time to fill the twi-buffer with data
	// the SCL-line will be held
}


void mtxComplete()
{
	// the data has been transmitted - you have to at least set the transceiver to slave mode
	twiBuffer[0] = ((0x10|slave)<<1) | 1;
	
	twiStart(2);
}

void mrxComplete(uint8_t bytes)
{
	// you have received your requested data, so handle it!
	uint8_t result = twiBuffer[1];
	
	uart_putc(result);
	
	if (result == 'h')
	  toggle_relais();
}

void srxComplete(uint8_t bytes)
{
	// you have received some data you did not request - so look if it's interesting!
	if (twiBuffer[0] == 0x20)
		twiBuffer[0] = 'h';
}

void stxComplete()
{
	// you sent the requested data
	// nothing has to be done, probably
	
	action = 1;
}

void gcxComplete(uint8_t bytes)
{
	// you have received a general call - so look if it's interesting
	// if you're not interested in general calls, deactivate them in the init function
}

void twiError(uint8_t errcode)
{
	// there was an error!
	// so check it out and perhaps you may want to try again to send your data
}




// ==================
// MAIN PROGRAM
//
int main( void )
{
  sei();
  
	// Coding of the two identical devices
  PORTB |= (1<<PB0);
  DDRB &= ~(1<<PB0);
  if (PINB & 1) {
  	master = 1;
  	slave = 0;
  }
  else {
  	master = 0;
  	slave = 1;
  }
  
  
  // Relais and Optocupler
  DDRD  = (1<<PD2); // relais = output
  PORTD = (1<<PD6); // pull-up for optocupler
  
  // Debugging
  DDRC  = 0x0F; // debug-output
  
  // init the twi
  twiInit(0x10|master, 0xFF, 0);
  
  
  start_system_timer();
  
  // help identify the devices
  if (!master)
  	toggle_relais();
  	
  uart_init(UART_BAUD_SELECT(9600, 16000000));
  uart_puts_p (PSTR("\n\ralive!"));
  
  
  // stop here
  for(;;)
  {
  	// do nothing. all time. cool, heh?
  }
}

