/************************************************
*
* JCH2k I2C Library for simultanious master & slave operation
*
* (c) 2006, 2007 J.Claudius Hagen, www.jch2k.de
*
* Revision          : $Revision: 637 $
* Date              : $Date: 2007-03-07 01:51:26 +0100 (Mi, 07 Mrz 2007) $
* Created at        : June 2007
* Updated by        : $Author: claudius $
*
* Details: http://www.jch2k.de (german)
* 
* License: This is free Software! You may share it, modify it and use it (commercially or not, i don't care)
* You only have to leave this License and the Author in this File and you have to share it!
*
**************************************************/


/****************************************************************************
  TWI Status/Control register definitions
****************************************************************************/
#define TWI_BUFFER_SIZE 40   // Set this to the largest message size that will be sent including address byte.
uint8_t twiBuffer[TWI_BUFFER_SIZE];

// read this if you want to know whether the transceiver is busy or not
// this is important! check twiBusy before you call twiStart()!!
uint8_t twiBusy;

/****************************************************************************
  Callback functions YOU HAVE TO IMPLEMENT
****************************************************************************/

// nothing has to be done
// but you can handle the received byte if you're in a hurry
extern void mrxByteReceived(uint8_t b);

// nothing has to be done
// but you can handle the received byte if you're in a hurry
extern void srxByteReceived(uint8_t b);

// nothing has to be done
// but you can handle the received byte if you're in a hurry
extern void gcxByteReceived(uint8_t b);

// here you got some time to fill the twi-buffer with data
// the SCL-line will be held
extern void stxStarted(void);

// the data has been transmitted - you have to at least set the transceiver to slave mode
extern void mtxComplete(void);

// you have received your requested data, so handle it!
extern void mrxComplete(uint8_t bytes);

// you sent the requested data
// nothing has to be done, probably	
extern void stxComplete(void);

// you have received some data you did not request - so look if it's interesting!
extern void srxComplete(uint8_t bytes);

// you have received a general call - so look if it's interesting
// if you're not interested in general calls, deactivate them in the init function
extern void gcxComplete(uint8_t bytes);

// there was an error!
// so check it out and perhaps you may want to try again to send your data
extern void twiError(uint8_t errcode);


/****************************************************************************
  Function definitions
****************************************************************************/
void twiStart(uint8_t msgSize);
void twiInit(uint8_t address, uint8_t bitrate, uint8_t gencall);


/****************************************************************************
  Bit and byte definitions
****************************************************************************/
#define TWI_READ_BIT  0       // Bit position for R/W bit in "address byte".

#define MASTER 12
#define SLAVE  13

#define BUSY 255


/****************************************************************************
  TWI State codes
****************************************************************************/
// General TWI Master staus codes                      
#define TWI_START                  0x08  // START has been transmitted  
#define TWI_REP_START              0x10  // Repeated START has been transmitted
#define TWI_ARB_LOST               0x38  // Arbitration lost

// TWI Master Transmitter staus codes                      
#define TWI_MTX_ADR_ACK            0x18  // SLA+W has been tramsmitted and ACK received
#define TWI_MTX_ADR_NACK           0x20  // SLA+W has been tramsmitted and NACK received 
#define TWI_MTX_DATA_ACK           0x28  // Data byte has been tramsmitted and ACK received
#define TWI_MTX_DATA_NACK          0x30  // Data byte has been tramsmitted and NACK received 

// TWI Master Receiver staus codes  
#define TWI_MRX_ADR_ACK            0x40  // SLA+R has been tramsmitted and ACK received
#define TWI_MRX_ADR_NACK           0x48  // SLA+R has been tramsmitted and NACK received
#define TWI_MRX_DATA_ACK           0x50  // Data byte has been received and ACK tramsmitted
#define TWI_MRX_DATA_NACK          0x58  // Data byte has been received and NACK tramsmitted

// TWI Slave Transmitter staus codes
#define TWI_STX_ADR_ACK            0xA8  // Own SLA+R has been received; ACK has been returned
#define TWI_STX_ADR_ACK_M_ARB_LOST 0xB0  // Arbitration lost in SLA+R/W as Master; own SLA+R has been received; ACK has been returned
#define TWI_STX_DATA_ACK           0xB8  // Data byte in TWDR has been transmitted; ACK has been received
#define TWI_STX_DATA_NACK          0xC0  // Data byte in TWDR has been transmitted; NOT ACK has been received
#define TWI_STX_DATA_ACK_LAST_BYTE 0xC8  // Last data byte in TWDR has been transmitted (TWEA = 0); ACK has been received

// TWI Slave Receiver staus codes
#define TWI_SRX_ADR_ACK            0x60  // Own SLA+W has been received ACK has been returned
#define TWI_SRX_ADR_ACK_M_ARB_LOST 0x68  // Arbitration lost in SLA+R/W as Master; own SLA+W has been received; ACK has been returned
#define TWI_SRX_GEN_ACK            0x70  // General call address has been received; ACK has been returned
#define TWI_SRX_GEN_ACK_M_ARB_LOST 0x78  // Arbitration lost in SLA+R/W as Master; General call address has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_ACK       0x80  // Previously addressed with own SLA+W; data has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_NACK      0x88  // Previously addressed with own SLA+W; data has been received; NOT ACK has been returned
#define TWI_SRX_GEN_DATA_ACK       0x90  // Previously addressed with general call; data has been received; ACK has been returned
#define TWI_SRX_GEN_DATA_NACK      0x98  // Previously addressed with general call; data has been received; NOT ACK has been returned
#define TWI_SRX_STOP_RESTART       0xA0  // A STOP condition or repeated START condition has been received while still addressed as Slave

// TWI Miscellaneous status codes
#define TWI_NO_STATE               0xF8  // No relevant state information available; TWINT = 0
#define TWI_BUS_ERROR              0x00  // Bus error due to an illegal START or STOP condition
